//
//  TestWizard_AppDelegate.m
//  TestWizard
//
//  Created by Thaddeus Cooper on 11/10/06.
//  Copyright Thaddeus O. Cooper 2006 . All rights reserved.
//

#import "TestWizard_AppDelegate.h"

@implementation TestWizard_AppDelegate


/**
    Returns the support folder for the application, used to store the Core Data
    store file.  This code uses a folder named "TestWizard" for
    the content, either in the NSApplicationSupportDirectory location or (if the
    former cannot be found), the system's temporary directory.
 */

- (NSString *)applicationSupportFolder {

    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
    NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : NSTemporaryDirectory();
    return [basePath stringByAppendingPathComponent:@"TestWizard"];
}


/**
    Creates, retains, and returns the managed object model for the application 
    by merging all of the models found in the application bundle and all of the 
    framework bundles.
 */
 
- (NSManagedObjectModel *)managedObjectModel {

    if (managedObjectModel != nil) {
        return managedObjectModel;
    }
	
    NSMutableSet *allBundles = [[NSMutableSet alloc] init];
    [allBundles addObject: [NSBundle mainBundle]];
    [allBundles addObjectsFromArray: [NSBundle allFrameworks]];
    
    managedObjectModel = [[NSManagedObjectModel mergedModelFromBundles: [allBundles allObjects]] retain];
    [allBundles release];
    
    return managedObjectModel;
}


/**
    Returns the persistent store coordinator for the application.  This 
    implementation will create and return a coordinator, having added the 
    store for the application to it.  (The folder for the store is created, 
    if necessary.)
 */

- (NSPersistentStoreCoordinator *) persistentStoreCoordinator {

    if (persistentStoreCoordinator != nil) {
        return persistentStoreCoordinator;
    }

    NSFileManager *fileManager;
    NSString *applicationSupportFolder = nil;
    NSURL *url;
    NSError *error;
    
    fileManager = [NSFileManager defaultManager];
    applicationSupportFolder = [self applicationSupportFolder];
    if ( ![fileManager fileExistsAtPath:applicationSupportFolder isDirectory:NULL] ) {
        [fileManager createDirectoryAtPath:applicationSupportFolder attributes:nil];
    }
    
    url = [NSURL fileURLWithPath: [applicationSupportFolder stringByAppendingPathComponent: @"TestWizard.xml"]];
    persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel: [self managedObjectModel]];
    if (![persistentStoreCoordinator addPersistentStoreWithType:NSXMLStoreType configuration:nil URL:url options:nil error:&error]){
        [[NSApplication sharedApplication] presentError:error];
    }    

    return persistentStoreCoordinator;
}


/**
    Returns the managed object context for the application (which is already
    bound to the persistent store coordinator for the application.) 
 */
 
- (NSManagedObjectContext *) managedObjectContext {

    if (managedObjectContext != nil) {
        return managedObjectContext;
    }

    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil) {
        managedObjectContext = [[NSManagedObjectContext alloc] init];
        [managedObjectContext setPersistentStoreCoordinator: coordinator];
    }
    
    return managedObjectContext;
}


/**
    Returns the NSUndoManager for the application.  In this case, the manager
    returned is that of the managed object context for the application.
 */
 
- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)window {
    return [[self managedObjectContext] undoManager];
}


/**
    Performs the save action for the application, which is to send the save:
    message to the application's managed object context.  Any encountered errors
    are presented to the user.
 */
 
- (IBAction) saveAction:(id)sender {

    NSError *error = nil;
    if (![[self managedObjectContext] save:&error]) {
        [[NSApplication sharedApplication] presentError:error];
    }
}


/**
    Implementation of the applicationShouldTerminate: method, used here to
    handle the saving of changes in the application managed object context
    before the application terminates.
 */
 
- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender {

    NSError *error;
    int reply = NSTerminateNow;
    
    if (managedObjectContext != nil) {
        if ([managedObjectContext commitEditing]) {
            if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error]) {
				
                // This error handling simply presents error information in a panel with an 
                // "Ok" button, which does not include any attempt at error recovery (meaning, 
                // attempting to fix the error.)  As a result, this implementation will 
                // present the information to the user and then follow up with a panel asking 
                // if the user wishes to "Quit Anyway", without saving the changes.

                // Typically, this process should be altered to include application-specific 
                // recovery steps.  

                BOOL errorResult = [[NSApplication sharedApplication] presentError:error];
				
                if (errorResult == YES) {
                    reply = NSTerminateCancel;
                } 

                else {
					
                    int alertReturn = NSRunAlertPanel(nil, @"Could not save changes while quitting. Quit anyway?" , @"Quit anyway", @"Cancel", nil);
                    if (alertReturn == NSAlertAlternateReturn) {
                        reply = NSTerminateCancel;	
                    }
                }
            }
        } 
        
        else {
            reply = NSTerminateCancel;
        }
    }
    
    return reply;
}


/**
    Implementation of dealloc, to release the retained variables.
 */
 
- (void) dealloc {

    [managedObjectContext release], managedObjectContext = nil;
    [persistentStoreCoordinator release], persistentStoreCoordinator = nil;
    [managedObjectModel release], managedObjectModel = nil;
    [super dealloc];
}

- (void) setupAuthenticationTypes {
	NSManagedObjectContext	*moc;
	NSManagedObject			*newData;
	NSArray					*authenticationTypes;
	int						index;
	int						count;
	NSDictionary			*dictionary;
	
	moc = [self managedObjectContext];
	authenticationTypes = [NSArray arrayWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"AuthenticationTypes" ofType:@"plist"]];
	count = [authenticationTypes count];
	
	for (index = 0; index < count; index++) {
		dictionary = [authenticationTypes objectAtIndex:index];
		newData    = [NSEntityDescription insertNewObjectForEntityForName:@"AuthenticationTypes"
												   inManagedObjectContext:moc];
		[newData setValue:[dictionary valueForKey:@"authenticationTypeString"] forKey:@"authenticationTypeString"];
		[newData setValue:[dictionary valueForKey:@"authenticationTypeValue"] forKey:@"authenticationTypeValue"];
		[newData setValue:[dictionary valueForKey:@"displayOrder"] forKey:@"displayOrder"];
	}
}

- (void) setupMailServers {
	NSManagedObjectContext	*moc;
	NSManagedObject			*newData;
	NSArray					*mailServers;
	int						count;
	int						index;
	NSDictionary			*dictionary;
	NSDictionary			*mailAppDefaults;
	
	moc = [self managedObjectContext];
	mailAppDefaults      = [[NSUserDefaults standardUserDefaults] persistentDomainForName:@"com.apple.mail"];
	mailServers          = [NSArray arrayWithArray:[mailAppDefaults valueForKey:@"DeliveryAccounts"]];
	
	count = [mailServers count];
	for (index = 0; index < count; index++) {
		dictionary = [mailServers objectAtIndex:index];
		if ([[dictionary valueForKey:@"AccountType"] isEqualToString:@"SMTPAccount"]) {
			newData = [NSEntityDescription insertNewObjectForEntityForName:@"SMTPServers"
													inManagedObjectContext:moc];
			[newData setValue:[NSNumber numberWithInt:index] forKey:@"displayOrder"];
			[newData setValue:[dictionary valueForKey:@"Hostname"] forKey:@"displayString"];
			[newData setValue:[dictionary valueForKey:@"Hostname"] forKey:@"hostname"];
			if (([dictionary valueForKey:@"PortNumber"] == nil) ||
				([[dictionary valueForKey:@"PortNumber"] intValue] == 0)) {
				[newData setValue:[NSNumber numberWithInt:25] forKey:@"serverPort"];		
			}
			else {
				[newData setValue:[NSNumber numberWithInt:[[dictionary valueForKey:@"PortNumber"] intValue]] forKey:@"serverPort"];				 
			}
			[newData setValue:[dictionary valueForKey:@"Username"] forKey:@"username"];
			[newData setValue:@"" forKey:@"password"];
			if ([dictionary valueForKey:@"SSLEnabled"] != nil) {
				if ([[dictionary valueForKey:@"SSLEnabled"] isEqualToString:@"YES"]) {
					[newData setValue:[NSNumber numberWithBool:YES] forKey:@"useSSL"];					
				}
				else {
					[newData setValue:[NSNumber numberWithBool:NO] forKey:@"useSSL"];
				}
			}
			else {
				[newData setValue:[NSNumber numberWithBool:NO] forKey:@"useSSL"];				
			}
		}
	}
	newData = [NSEntityDescription insertNewObjectForEntityForName:@"SMTPServers"
											inManagedObjectContext:moc];
	[newData setValue:@"Custom" forKey:@"displayString"];
	[newData setValue:[NSNumber numberWithInt:index] forKey:@"displayOrder"];
}

- (void) setupEmailAccounts {
	NSManagedObjectContext	*moc;
	NSManagedObject			*newData;
	NSArray					*emailAddresses;
	int						count;
	int						index;
	NSDictionary			*dictionary;
	NSDictionary			*mailAppDefaults;
	
	moc             = [self managedObjectContext];
	mailAppDefaults = [[NSUserDefaults standardUserDefaults] persistentDomainForName:@"com.apple.mail"];
	emailAddresses  = [NSArray arrayWithArray:[mailAppDefaults valueForKey:@"MailAccounts"]];
	
	count = [emailAddresses count];
	for (index = 0; index < count; index++) {
		dictionary = [emailAddresses objectAtIndex:index];
		if (![[dictionary valueForKey:@"AccountType"] isEqualToString:@"LocalAccount"]) {
			newData = [NSEntityDescription insertNewObjectForEntityForName:@"EmailAddresses"
													inManagedObjectContext:moc];
			[newData setValue:[NSNumber numberWithInt:index] forKey:@"displayOrder"];
			[newData setValue:[dictionary valueForKey:@"FullUserName"] forKey:@"fullName"];
			[newData setValue:[dictionary valueForKey:@"AccountName"] forKey:@"accountName"];
			if ([[dictionary valueForKey:@"AccountType"] isEqualToString:@"iToolsAccount"]) {
				[newData setValue:[dictionary valueForKey:@"AccountName"] forKey:@"emailAddress"];
			}
			else if ([[dictionary valueForKey:@"AccountType"] isEqualToString:@"POPAccount"]) {
				[newData setValue:[[dictionary valueForKey:@"EmailAddresses"] objectAtIndex:0] forKey:@"emailAddress"];
			}
			else if ([[dictionary valueForKey:@"AccountType"] isEqualToString:@"IMapAccount"]) {
				[newData setValue:[[dictionary valueForKey:@"EmailAddresses"] objectAtIndex:0] forKey:@"emailAddress"];
			}
			else if ([[dictionary valueForKey:@"AccountType"] isEqualToString:@"ExchangeAccount"]) {
				[newData setValue:[[dictionary valueForKey:@"EmailAddresses"] objectAtIndex:0] forKey:@"emailAddress"];
			}
		}
	}
	newData = [NSEntityDescription insertNewObjectForEntityForName:@"EmailAddresses"
											inManagedObjectContext:moc];
	[newData setValue:@"Custom" forKey:@"accountName"];
	[newData setValue:[NSNumber numberWithInt:index] forKey:@"displayOrder"];
}

- (void) awakeFromNib {
	if (!initialized) {
		[self setupAuthenticationTypes];
		[self setupEmailAccounts];
		[self setupMailServers];
		initialized = YES;
	}
}

- (NSArray *) sortByDisplayOrderSortDescriptor {
	NSSortDescriptor	*sortDescriptor;
	NSArray				*anArray;
	
	sortDescriptor = [[[NSSortDescriptor alloc] initWithKey:@"displayOrder" ascending:YES] autorelease];
	anArray        = [NSArray arrayWithObject:sortDescriptor];
	return(anArray);
}

- (IBAction) saveSelectedDataToPreferences:(id)sender {
	NSManagedObject	*anAddress;
	
	anAddress = [[emailAddressArrayController selectedObjects] objectAtIndex:0];
	NSLog(@"%@:%@ | %@", [anAddress valueForKey:@"accountName"], [anAddress valueForKey:@"fullName"], [anAddress valueForKey:@"emailAddress"]);
}

- (NSNumber *) customEmailAddressSelected {
	NSManagedObject	*anAddress;
	NSNumber		*rc;
	
	if ([[emailAddressArrayController selectedObjects] count] > 0) {
		anAddress = [[emailAddressArrayController selectedObjects] objectAtIndex:0];
		if ([[anAddress valueForKey:@"accountName"] isEqualToString:@"Custom"]) {
			rc = [NSNumber numberWithBool:YES];
		}
		else {
			rc = [NSNumber numberWithBool:NO];
		}		
	}
	else {
		rc = [NSNumber numberWithBool:NO];		
	}
	return(rc);
}

- (NSNumber *) customSMTPServerSelected {
	NSManagedObject	*aServer;
	NSNumber		*rc;
	
	if ([[smtpServersArrayController selectedObjects] count] > 0) {
		aServer = [[smtpServersArrayController selectedObjects] objectAtIndex:0];
		if ([[aServer valueForKey:@"displayString"] isEqualToString:@"Custom"]) {
			rc = [NSNumber numberWithBool:YES];
		}
		else {
			rc = [NSNumber numberWithBool:NO];
		}		
	}
	else {
		rc = [NSNumber numberWithBool:NO];		
	}
	return(rc);
}

- (NSString *) passwordLabel {
	return(@"Password:");
}
@end
